@@ -21,7 +21,10 @@ y <- x + 0.25*x2 + 0.5*x3 + id.eff[id] + firm.eff[firm] + shoe.eff[shoe] + shirt
 summary(est <- felm(y ~ x+x2 + x3 + G(id) + G(firm) + G(shoe) + G(shirt)))
 cat('Components:',nlevels(est$cfactor),'largest:',sum(est$cfactor == '1'),'\n')
 ## extract the group fixed effects
-for(ef in c('ln','ref')) {
+  ## verify that id and firm coefficients are 1
+options(scipen=8)
+
+for(ef in c('ln','ref','zm','zm2')) {
   fe <- getfe(est,ef=ef)
   ## merge back
 
@@ -29,8 +32,10 @@ for(ef in c('ln','ref')) {
   firmeff <- fe[paste('firm',firm,sep='.'),'effect']
   shoeeff <- fe[paste('shoe',shoe,sep='.'),'effect']
   shirteff <- fe[paste('shirt',shirt,sep='.'),'effect']
-
-  ## verify that id and firm coefficients are 1
-  options(scipen=8)
-  print(summary(lm(y ~ x + x2 + x3 + ideff + firmeff + shoeeff +shirteff -1),digits=8))
+  if(ef %in% c('zm','zm2')) {
+    icpt <- fe[paste('icpt',1:nlevels(est$cfactor),sep='.'),'effect'][est$cfactor]
+    print(summary(lm(y ~ x + x2 + x3 + ideff + firmeff + shoeeff +shirteff + icpt-1),digits=8))
+  } else {
+    print(summary(lm(y ~ x + x2 + x3 + ideff + firmeff + shoeeff +shirteff-1),digits=8))
+  }
 }
